//////////////////////////////////////////////////////////////////////////////
// suseCrystalconfig.cc
// -------------------
// Config module for suseCrystal window decoration
// -------------------
// Copyright (c) 2003 David Johnson <david@usermode.org>
// Please see the header file for copyright and license information.
//////////////////////////////////////////////////////////////////////////////


#include "suseCrystalconfig.h"
#include "configdialog.h"

//////////////////////////////////////////////////////////////////////////////
// suseCrystalConfig()
// -------------
// Constructor

suseCrystalConfig::suseCrystalConfig(KConfig* config, QWidget* parent)
    : QObject(parent), config_(0), dialog_(0)
{

    // create the configuration object
    config_ = new KConfig("kwinsuseCrystalrc");
    KGlobal::locale()->insertCatalogue("kwin_suseCrystal_config");

    // create and show the configuration dialog
    dialog_ = new ConfigDialog(parent);
    dialog_->show();
 
    // load the configuration
    load(config_);

    // setup the connections
    connect(dialog_->titlealign, SIGNAL(clicked(int)),
            this, SLOT(selectionChanged(int)));

}

//////////////////////////////////////////////////////////////////////////////
// ~suseCrystalConfig()
// --------------
// Destructor

suseCrystalConfig::~suseCrystalConfig()
{
    if (dialog_) delete dialog_;
    if (config_) delete config_;
}

//////////////////////////////////////////////////////////////////////////////
// selectionChanged()
// ------------------
// Selection has changed

void suseCrystalConfig::selectionChanged(int)
{
    emit changed();
}

//////////////////////////////////////////////////////////////////////////////
// load()
// ------
// Load configuration data

void suseCrystalConfig::load(KConfig*)
{
    config_->setGroup("General");
    
    QString value = config_->readEntry("TitleAlignment", "AlignHCenter");
    QRadioButton *button = (QRadioButton*)dialog_->titlealign->child(value);
    if (button) button->setChecked(true);
}

//////////////////////////////////////////////////////////////////////////////
// save()
// ------
// Save configuration data

void suseCrystalConfig::save(KConfig*)
{
    config_->setGroup("General");

    QRadioButton *button = (QRadioButton*)dialog_->titlealign->selected();
    if (button) config_->writeEntry("TitleAlignment", QString(button->name()));
    config_->sync();
}

//////////////////////////////////////////////////////////////////////////////
// defaults()
// ----------
// Set configuration defaults

void suseCrystalConfig::defaults()
{
    QRadioButton *button =
        (QRadioButton*)dialog_->titlealign->child("AlignHCenter");
    if (button) button->setChecked(true);
}

//////////////////////////////////////////////////////////////////////////////
// Plugin Stuff                                                             //
//////////////////////////////////////////////////////////////////////////////

extern "C"
{
    QObject* allocate_config(KConfig* config, QWidget* parent) {
        return (new suseCrystalConfig(config, parent));
    }
}

#include "suseCrystalconfig.moc"
