//////////////////////////////////////////////////////////////////////////////
// suseCrystalclient.cc
// -------------------
// suseCrystal window decoration for KDE
// -------------------
// Copyright (c) 2003, 2004 David Johnson
// Please see the header file for copyright and license information.
//////////////////////////////////////////////////////////////////////////////



// xpm

	// buttoons

#include "xpm/buttons/blackClose.xpm"
#include "xpm/buttons/whiteClose.xpm"
#include "xpm/buttons/blackMinimize.xpm"
#include "xpm/buttons/whiteMinimize.xpm"
#include "xpm/buttons/blackMaximize.xpm"
#include "xpm/buttons/whiteMaximize.xpm"
#include "xpm/buttons/blackRestore.xpm"
#include "xpm/buttons/whiteRestore.xpm"
#include "xpm/buttons/blackHelp.xpm"
#include "xpm/buttons/whiteHelp.xpm"
#include "xpm/buttons/blackSticky.xpm"
#include "xpm/buttons/whiteSticky.xpm"
#include "xpm/buttons/blackUnsticky.xpm"
#include "xpm/buttons/whiteUnsticky.xpm"

	// borders

#include "xpm/borders/t.xpm"
#include "xpm/borders/b.xpm"
#include "xpm/borders/l.xpm"
#include "xpm/borders/r.xpm"
#include "xpm/borders/tl.xpm"
#include "xpm/borders/tm.xpm"
#include "xpm/borders/button.xpm"
#include "xpm/borders/cbl.xpm"
#include "xpm/borders/cbr.xpm"
#include "xpm/borders/ctl.xpm"
#include "xpm/borders/ctr.xpm"

// qt

#include <qpoint.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qtooltip.h>
#include <qcolor.h>

// kde

#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <klocale.h>

#include "suseCrystalclient.h"

using namespace suseCrystal;

// global constants

static const int BUTTONSIZE = 30;
static const int DECOSIZE = 16;
static const int TITLESIZE = 30;
static const int FRAMESIZE = 8;

// button decorations


static QImage blackCloseImg = QImage( blackClose_xpm );
static QImage whiteCloseImg = QImage( whiteClose_xpm );
static QImage blackMinimizeImg = QImage( blackMinimize_xpm );
static QImage whiteMinimizeImg = QImage( whiteMinimize_xpm );
static QImage blackMaximizeImg = QImage( blackMaximize_xpm );
static QImage whiteMaximizeImg = QImage( whiteMaximize_xpm );
static QImage blackRestoreImg = QImage( blackRestore_xpm );
static QImage whiteRestoreImg = QImage( whiteRestore_xpm );
static QImage blackHelpImg = QImage( blackHelp_xpm );
static QImage whiteHelpImg = QImage( whiteHelp_xpm );
static QImage blackStickyImg = QImage( blackSticky_xpm );
static QImage whiteStickyImg = QImage( whiteSticky_xpm );
static QImage blackUnstickyImg = QImage( blackUnsticky_xpm );
static QImage whiteUnstickyImg = QImage( whiteUnsticky_xpm );

// border decorations 

static QPixmap  t = QPixmap ( t_xpm);
static QPixmap  b = QPixmap ( b_xpm);
static QPixmap  l = QPixmap ( l_xpm);
static QPixmap  r = QPixmap ( r_xpm);
static QPixmap  tl = QPixmap ( tl_xpm);
static QPixmap  tm = QPixmap ( tm_xpm);
static QPixmap  button = QPixmap ( button_xpm);
static QPixmap  cbl = QPixmap ( cbl_xpm);
static QPixmap  cbr = QPixmap ( cbr_xpm);
static QPixmap  ctl = QPixmap ( ctl_xpm);
static QPixmap  ctr = QPixmap ( ctr_xpm);
 

// suseCrystalFactory *************************************************************
//*****************************************************************************

bool suseCrystalFactory::initialized_ = false;
Qt::AlignmentFlags suseCrystalFactory::titlealign_ = Qt::AlignHCenter;

extern "C" KDecorationFactory* create_factory()
{
	return new suseCrystal::suseCrystalFactory();
}

//////////////////////////////////////////////////////////////////////////////
// suseCrystalFactory()
// ----------------
// Constructor

suseCrystalFactory::suseCrystalFactory()
{
	readConfig();
	initialized_ = true;
}

///////////////////////////////////////////ButtonType///////////////////////////////////
// ~suseCrystalFactory()
// -----------------
// Destructor

suseCrystalFactory::~suseCrystalFactory()
{
	initialized_ = false;
}

//////////////////////////////////////////////////////////////////////////////
// createDecoration()
// -----------------
// Create the decoration

KDecoration* suseCrystalFactory::createDecoration( KDecorationBridge* b )
{
	return new suseCrystalClient( b, this );
}

/////////////////////////////////////////////////ButtonType/////////////////////////////
// reset()
// -------
// Reset the handler. Returns true if decorations need to be remade, false if
// only a repaint is necessary

bool suseCrystalFactory::reset( unsigned long changed )
{
	// read in the configuration
	initialized_ = false;
	bool confchange = readConfig();
	initialized_ = true;

	if ( confchange || ( changed & ( SettingDecoration | SettingButtons | SettingBorder ) ) )
	{
		return true;
	}
	else
	{
		resetDecorations( changed );
		return false;
	}
}

//////////////////////////////////////////////////////////////////////////////
// readConfig()
// ------------
// Read in the configuration file

bool suseCrystalFactory::readConfig()
{
	// create a config object
	KConfig config( "kwinsuseCrystalrc" );
	config.setGroup( "General" );

	// grab settings
	Qt::AlignmentFlags oldalign = titlealign_;
	QString value = config.readEntry( "TitleAlignment", "AlignHCenter" );
	if ( value == "AlignLeft" )
		titlealign_ = Qt::AlignLeft;
	else if ( value == "AlignHCenter" )
		titlealign_ = Qt::AlignHCenter;
	else if ( value == "AlignRight" )
		titlealign_ = Qt::AlignRight;

	if ( oldalign == titlealign_ )
		return false;
	else
		return true;
}

// end suseCrystalFactory *********************************************************
//*****************************************************************************




// suseCrystalButton **************************************************************
//*****************************************************************************

//////////////////////////////////////////////////////////////////////////////
// suseCrystalButton()
// ---------------ButtonType
// Constructor



suseCrystalButton::suseCrystalButton( suseCrystalClient *parent, const char *name,
                              const QString& tip, ButtonType type,
							  bool bitmapFlag,
                              QImage *bImage, QImage *wImage,bool isRight )
		: QButton( parent->widget(), name ), client_( parent ), type_( type ),
		lastmouse_( 0 ), hover( false ), bImage_( 0 ), wImage_(0), isRight_( isRight )
{
	setBackgroundMode( NoBackground );
	setFixedSize( BUTTONSIZE - 6, BUTTONSIZE );
	setCursor( arrowCursor );

	if ( bitmapFlag )
	{
		setBitmap( bitmapFlag, bImage, wImage );
	}
	QToolTip::add
		( this, tip );

}


suseCrystalButton::~suseCrystalButton()
{
	if (bImage_)
		delete bImage_;
	if (wImage_)
		delete wImage_;
}

//////////////////////////////////////////////////////////////////////////////
// setBitmap()
// -----------
// Set the button decoration

void suseCrystalButton::setBitmap( bool bitmapFlag, QImage *bImage, QImage *wImage )
{
	if ( !bitmapFlag )
		return ; // no bitmap, probably the menu button

	if (bImage_)
		delete bImage_;
	if (bImage_)	
		delete wImage_;
		
	bImage_ = new QImage( *bImage );
	wImage_ = new QImage( *wImage );
	
	repaint( false );
}

//////////////////////////////////////////////////////////////////////////////
// sizeHint()
// ----------
// Return size hint#include "xpm/close_down.xpm"

QSize suseCrystalButton::sizeHint() const
{
	return QSize( BUTTONSIZE - 6, BUTTONSIZE );
}

//////////////////////////////////////////////////////////////////////////////
// enterEvent()
// ------------
// Mouse has entered the button

void suseCrystalButton::enterEvent( QEvent *e )
{
	// if we wanted to do mouseovers, we would keep track of it here
	QButton::enterEvent( e );

	hover = true;
	repaint( false );
}

//////////////////////////////////////////////////////////////////////////////
// leaveEvent()
// ------------
// Mouse has left the button

void suseCrystalButton::leaveEvent( QEvent *e )
{
	// if we wanted to do mouseovers, we would keep track of it here
	QButton::leaveEvent( e );

	hover = false;
	repaint( false );
}
//////////////////////////////////////////////////////////////////////////////
// mousePressEvent()
// -----------------
// Button has been pressed

void suseCrystalButton::mousePressEvent( QMouseEvent* e )
{
	lastmouse_ = e->button();

	// translate and pass on mouse event
	int button = LeftButton;
	if ( ( type_ != ButtonMax ) && ( e->button() != LeftButton ) )
	{
		button = NoButton; // middle & right buttons inappropriate
	}
	QMouseEvent me( e->type(), e->pos(), e->globalPos(),
	                button, e->state() );
	QButton::mousePressEvent( &me );
}

//////////////////////////////////////////////////////////////////////////////
// mouseReleaseEvent()
// -----------------
// Button has been released

void suseCrystalButton::mouseReleaseEvent( QMouseEvent* e )
{
	lastmouse_ = e->button();

	// translate and pass on mouse event
	int button = LeftButton;
	if ( ( type_ != ButtonMax ) && ( e->button() != LeftButton ) )
	{
		button = NoButton; // middle & right buttons inappropriate
	}
	QMouseEvent me( e->type(), e->pos(), e->globalPos(), button, e->state() );
	QButton::mouseReleaseEvent( &me );
}

//////////////////////////////////////////////////////////////////////////////
// drawButton()
// ------------
// Draw the button

void suseCrystalButton::drawButton( QPainter *painter )
{
	if ( !suseCrystalFactory::initialized() )
		return ;

	int dx, dy;

	QRect frame( 0, 0, width(), height() );
	
	if( !isRight_)
		painter->drawTiledPixmap( frame, QPixmap( button ) );
	else
		painter->drawTiledPixmap( frame, QPixmap( tm ) );

	if ( type_ == ButtonMenu )
	{
		// we paint the mini icon (which is 16 pixels high)
		dx = ( width() - 16 ) / 2;
		dy = ( height() - 16 ) / 2;
		if ( isDown() )
		{
			dx++;
			dy++;
		}
		painter->drawPixmap( dx, dy, client_->icon().pixmap( QIconSet::Small, QIconSet::Normal ) );
	}
	else if ( bImage_ && wImage_)
	{
		// otherwise we paint the deco
		dx = ( width() - DECOSIZE ) / 2;
		dy = ( height() - DECOSIZE ) / 2;
		
		if ( isDown() )
		{
			dx++;
			dy++;
		}
		
		if ( hover )
		{
			*wImage_ = wImage_->smoothScale( width() - 6, height() - 12 );
			painter->drawImage( dx, dy, *wImage_ );
		}
		else
		{
		
			*bImage_ = bImage_->smoothScale( width() - 6, height() - 12 );
			painter->drawImage( dx, dy, *bImage_ );
		}
	}
}

// end suseCrystalButton **********************************************************
//*****************************************************************************



				
// suseCrystalClient **************************************************************
//*****************************************************************************

// ---------------
// Constructor

suseCrystalClient::suseCrystalClient( KDecorationBridge *b, KDecorationFactory *f )
		: KDecoration( b, f )
{
	;
}

suseCrystalClient::~suseCrystalClient()
{
	for ( int n = 0; n < ButtonTypeCount; n++ )
	{
		if ( button[ n ] )
			delete button[ n ];
	}
}

//////////////////////////////////////////////////////////////////////////////
// init()
// ------
// Actual initializer for class


void suseCrystalClient::init()
{
	createMainWidget( WResizeNoErase | WRepaintNoErase );
	widget() ->installEventFilter( this );

	// for flicker-free redraws
	widget() ->setBackgroundMode( NoBackground );

	// setup layout
	QGridLayout *mainlayout = new QGridLayout( widget(), 4, 3 ); // 4x3 grid
	QHBoxLayout *titlelayout = new QHBoxLayout();
	titlebar_ = new QSpacerItem( 1, TITLESIZE, QSizePolicy::Expanding,
	                             QSizePolicy::Fixed );

	mainlayout->setResizeMode( QLayout::FreeResize );
	mainlayout->addRowSpacing( 0, 0 );
	mainlayout->addRowSpacing( 3, FRAMESIZE );
	mainlayout->addColSpacing( 0, FRAMESIZE );
	mainlayout->addColSpacing( 2, FRAMESIZE );
	// 	mainlayout->addRowSpacing( 0, FRAMESIZE  );
	// 	mainlayout->addRowSpacing( 3, ( FRAMESIZE ) );
	// 	mainlayout->addColSpacing( 0, FRAMESIZE );
	// 	mainlayout->addColSpacing( 2, FRAMESIZE );

	mainlayout->addLayout( titlelayout, 1, 1 );
	if ( isPreview() )
	{
		mainlayout->addWidget(
		    new QLabel( i18n( "<b><center>suseCrystal preview</center></b>" ),
		                widget() ), 2, 1 );
	}
	else
	{
		mainlayout->addItem( new QSpacerItem( 0, 0 ), 2, 1 );
	}

	// the window should stretch
	mainlayout->setRowStretch( 2, 10 );
	mainlayout->setColStretch( 1, 10 );

	// setup titlebar buttons
	for ( int n = 0; n < ButtonTypeCount; n++ )
		button[ n ] = 0;
		
	addButtons( titlelayout, options() ->titleButtonsLeft(), 0 );
	titlelayout->addItem( titlebar_ );
	addButtons( titlelayout, options() ->titleButtonsRight(), 1 );
}

//////////////////////////////////////////////////////////////////////////////
// addButtons()
// ------------
// Add buttons to title layout
 
void suseCrystalClient::addButtons( QBoxLayout *layout, const QString& s, bool isRight )
{
	
	QString tip;
	QImage bImage;
	QImage wImage;
	rightButtonCount = 0;

	if ( s.length() > 0 )
	{
		for ( unsigned n = 0; n < s.length(); n++ )
		{
			switch ( s[ n ] )
			{
			case 'M':               // Menu button
				if ( !button[ ButtonMenu ] )
				{
					button[ ButtonMenu ] =
					    new suseCrystalButton( this, "menu", i18n( "Menu" ),
					                       ButtonMenu, 0, 0, 0, isRight );
					connect( button[ ButtonMenu ], SIGNAL( pressed() ),
					         this, SLOT( menuButtonPressed() ) );
					layout->addWidget( button[ ButtonMenu ] );
					if ( isRight )
						rightButtonCount++;
				}
				break;

			case 'S':               // Sticky button
				if ( !button[ ButtonSticky ] )
				{
					if ( isOnAllDesktops() )
					{
						bImage = QImage( blackStickyImg );
						wImage = QImage( whiteStickyImg );
						tip = i18n( "Sticky" );
					}
					else
					{
						bImage = QImage( blackUnstickyImg );
						wImage = QImage( whiteUnstickyImg );
						tip = i18n( "Un-Sticky" );
					}
					button[ ButtonSticky ] =
					    new suseCrystalButton( this, "sticky", tip,
					                       ButtonSticky, 1, &bImage, &wImage, isRight );
					connect( button[ ButtonSticky ], SIGNAL( clicked() ),
					         this, SLOT( toggleOnAllDesktops() ) );
					layout->addWidget( button[ ButtonSticky ] );
					if ( isRight )
						rightButtonCount++;
				}
				break;

			case 'H':               // Help button
				if ( ( !button[ ButtonHelp ] ) && providesContextHelp() )
				{
					button[ ButtonHelp ] =
					    new suseCrystalButton( this, "help", i18n( "Help" ),
					                       ButtonHelp, 1, &blackHelpImg, &whiteHelpImg, isRight );
					connect( button[ ButtonHelp ], SIGNAL( clicked() ),
					         this, SLOT( showContextHelp() ) );
					layout->addWidget( button[ ButtonHelp ] );
					if ( isRight )
						rightButtonCount++;
				}
				break;

			case 'I':               // Minimize button
				if ( ( !button[ ButtonMin ] ) && isMinimizable() )
				{
					button[ ButtonMin ] =
					    new suseCrystalButton( this, "iconify", i18n( "Minimize" ),
					                       ButtonMin, 1, &blackMinimizeImg, &whiteMinimizeImg, isRight );
					connect( button[ ButtonMin ], SIGNAL( clicked() ),
					         this, SLOT( minimize() ) );
					layout->addWidget( button[ ButtonMin ] );
					if ( isRight )
						rightButtonCount++;
				}
				break;

			case 'A':               // Maximize button
				if ( ( !button[ ButtonMax ] ) && isMaximizable() )
				{
					if ( maximizeMode() == MaximizeFull )
					{
						bImage = QImage( blackRestoreImg );
						wImage = QImage( whiteRestoreImg );
						tip = i18n( "Restore" );
					}
					else
					{
						bImage = QImage( blackMaximizeImg );
						wImage = QImage( whiteMaximizeImg );

						tip = i18n( "Maximize" );
					}
					button[ ButtonMax ] =
					    new suseCrystalButton( this, "maximize", tip,
					                       ButtonMax, 1, &bImage, &wImage, isRight );
					connect( button[ ButtonMax ], SIGNAL( clicked() ),
					         this, SLOT( maxButtonPressed() ) );
					layout->addWidget( button[ ButtonMax ] );
					if ( isRight )
						rightButtonCount++;
				}
				break;

			case 'X':               // Close button
				if ( ( !button[ ButtonClose ] ) && isCloseable() )
				{
					button[ ButtonClose ] =
					    new suseCrystalButton( this, "close", i18n( "Close" ),
					                       ButtonClose, 1, &blackCloseImg, &whiteCloseImg, isRight );
					connect( button[ ButtonClose ], SIGNAL( clicked() ),
					         this, SLOT( closeWindow() ) );
					layout->addWidget( button[ ButtonClose ] );
					if ( isRight )
						rightButtonCount++;
				}
				break;

			case '_':               // Spacer item
				layout->addSpacing( FRAMESIZE );
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// doShape()
// ---------
// shape Corners

void suseCrystalClient::doShape()
{
	QRegion mask( 0, 0, width(), height() );


	//  QRegion mask(width(), titlebar_->geometry().height());
	int r( width() );
	int b( height() - 1 );
	int rightSpacer = ( ( rightButtonCount * (BUTTONSIZE - 6) ) + FRAMESIZE );
	
	// Remove top-left corner.
	mask -= QRegion( 0, 0, 5, 1 );
	mask -= QRegion( 0, 1, 3, 1 );
	mask -= QRegion( 0, 2, 2, 1 );
	mask -= QRegion( 0, 3, 1, 2 );

	// Remove top-right corner.
 	mask -= QRegion( r - (rightSpacer + 21) , 0, (rightSpacer + 21) , 1 );
 	mask -= QRegion( r - (rightSpacer + 20) , 1, (rightSpacer + 20) , 1 );
	mask -= QRegion( r - (rightSpacer + 19) , 2, (rightSpacer + 19) , 1 );
	mask -= QRegion( r - (rightSpacer + 18) , 3, (rightSpacer + 18) , 1 );
	mask -= QRegion( r - (rightSpacer + 18) , 4, (rightSpacer + 18) , 1 );
							
	mask -= QRegion( r - 5, 5, 5, 1 );
	mask -= QRegion( r - 3, 6, 3, 1 );
	mask -= QRegion( r - 2, 7, 2, 1 );
	mask -= QRegion( r - 1, 8, 1, 2 );


	// Remove bottom-left corner.
	mask -= QRegion( 0, b - 0, 5, 1 );
	mask -= QRegion( 0, b - 1, 3, 1 );
	mask -= QRegion( 0, b - 2, 2, 1 );
	mask -= QRegion( 0, b - 4, 1, 2 );

	// Remove bottom-right corner.
	mask -= QRegion( r - 5, b - 0, 5, 1 );
	mask -= QRegion( r - 3, b - 1, 3, 1 );
	mask -= QRegion( r - 2, b - 2, 2, 1 );
	mask -= QRegion( r - 1, b - 4, 1, 2 );

	setMask( mask );
}

//////////////////////////////////////////////////////////////////////////////
// activeChange()
// --------------
// window active state has changed

void suseCrystalClient::activeChange()
{
	for ( int n = 0; n < ButtonTypeCount; n++ )
		if ( button[ n ] )
			button[ n ] ->reset();
	widget() ->repaint( false );
}

//////////////////////////////////////////////////////////////////////////////
// captionChange()
// ---------------
// The title has changed

void suseCrystalClient::captionChange()
{
	widget() ->repaint( titlebar_->geometry(), false );
}

//////////////////////////////////////////////////////////////////////////////
// desktopChange()
// ---------------
// Called when desktop/sticky changes

void suseCrystalClient::desktopChange()
{

	bool d = isOnAllDesktops();
	if ( button[ ButtonSticky ] )
	{
		button[ ButtonSticky ] ->setBitmap( 1, d ? &blackStickyImg : &blackUnstickyImg, d ? &whiteStickyImg : &whiteUnstickyImg );
		
// 				button[ ButtonSticky ] ->setBitmap( 1, d ? &blackUnstickyImg : &blackStickyImg, d ? &whiteUnstickyImg : &whiteStickyImg );
				
		QToolTip::remove
			( button[ ButtonSticky ] );
		QToolTip::add
			( button[ ButtonSticky ], d ? i18n( "Sticky" ) : i18n( "Un-Sticky" ) );
	}
}

//////////////////////////////////////////////////////////////////////////////
// iconChange()
// ------------
// The title has changed

void suseCrystalClient::iconChange()
{
	if ( button[ ButtonMenu ] )
	{
		button[ ButtonMenu ] ->setBitmap( 0 ,0 ,0);
		button[ ButtonMenu ] ->repaint( false );
	}
}

//////////////////////////////////////////////////////////////////////////////
// maximizeChange()
// ----------------
// Maximized state has changed

void suseCrystalClient::maximizeChange()
{
	bool m = ( maximizeMode() == MaximizeFull );
	if ( button[ ButtonMax ] )
	{
		button[ ButtonMax ] ->setBitmap( 1, m ? &blackRestoreImg : &blackMaximizeImg, 
		m ? &whiteRestoreImg : &whiteMaximizeImg );
		QToolTip::remove
			( button[ ButtonMax ] );
		QToolTip::add
			( button[ ButtonMax ], m ? i18n( "Restore" ) : i18n( "Maximize" ) );
	}
}

//////////////////////////////////////////////////////////////////////////////
// shadeChange()
// -------------
// Called when window shading changes

void suseCrystalClient::shadeChange()
{
	;
}

//////////////////////////////////////////////////////////////////////////////
// borders()
// ----------
// Get the size of the borders

void suseCrystalClient::borders( int &l, int &r, int &t, int &b ) const
{
	l = r = FRAMESIZE;
	t = TITLESIZE;
	b = FRAMESIZE;
}

//////////////////////////////////////////////////////////////////////////////
// resize()
// --------
// Called to resize the window

void suseCrystalClient::resize( const QSize &size )
{
	widget() ->resize( size );
}

//////////////////////////////////////////////////////////////////////////////
// minimumSize()
// -------------
// Return the minimum allowable size for this window

QSize suseCrystalClient::minimumSize() const
{
	return widget() ->minimumSize();
}

//////////////////////////////////////////////////////////////////////////////
// mousePosition()
// ---------------
// Return logical mouse position

KDecoration::Position suseCrystalClient::mousePosition( const QPoint &point ) const
{
	const int corner = 24;
	Position pos;

	if ( point.y() <= FRAMESIZE )
	{
		// inside top frame
		if ( point.x() <= corner )
			pos = PositionTopLeft;
		else if ( point.x() >= ( width() - corner ) )
			pos = PositionTopRight;
		else
			pos = PositionTop;
	}
	else if ( point.y() >= ( height() - ( FRAMESIZE ) ) )
	{
		// inside handle
		if ( point.x() <= corner )
			pos = PositionBottomLeft;
		else if ( point.x() >= ( width() - corner ) )
			pos = PositionBottomRight;
		else
			pos = PositionBottom;
	}
	else if ( point.x() <= FRAMESIZE )
	{
		// on left frame
		if ( point.y() <= corner )
			pos = PositionTopLeft;
		else if ( point.y() >= ( height() - corner ) )
			pos = PositionBottomLeft;
		else
			pos = PositionLeft;
	}
	else if ( point.x() >= width() - FRAMESIZE )
	{
		// on right frame
		if ( point.y() <= corner )
			pos = PositionTopRight;
		else if ( point.y() >= ( height() - corner ) )
			pos = PositionBottomRight;
		else
			pos = PositionRight;
	}
	else
	{
		// inside the frame
		pos = PositionCenter;
	}
	return pos;
}

//////////////////////////////////////////////////////////////////////////////
// eventFilter()
// -------------
// Event filter

bool suseCrystalClient::eventFilter( QObject *obj, QEvent *e )
{
	if ( obj != widget() )
		return false;

	switch ( e->type() )
	{
	case QEvent::MouseButtonDblClick:
		{
			mouseDoubleClickEvent( static_cast<QMouseEvent *>( e ) );
			return true;
		}
	case QEvent::MouseButtonPress:
		{
			processMousePressEvent( static_cast<QMouseEvent *>( e ) );
			doShape();
			return true;
		}
	case QEvent::Paint:
		{
			paintEvent( static_cast<QPaintEvent *>( e ) );
			return true;
		}
	case QEvent::Resize:
		{
			resizeEvent( static_cast<QResizeEvent *>( e ) );
			return true;
		}
	case QEvent::Show:
		{
			showEvent( static_cast<QShowEvent *>( e ) );
			return true;
		}
	default:
		{
			return false;
		}
	}

	return false;
}

//////////////////////////////////////////////////////////////////////////////
// mouseDoubleClickEvent()
// -----------------------
// Doubleclick on title

void suseCrystalClient::mouseDoubleClickEvent( QMouseEvent *e )
{
	if ( titlebar_->geometry().contains( e->pos() ) )
		titlebarDblClickOperation();
}

//////////////////////////////////////////////////////////////////////////////
// paintEvent()
// ------------
// Repaint the window


void suseCrystalClient::paintEvent( QPaintEvent* )
{
	if ( !suseCrystalFactory::initialized() )
		return ;

	QPainter painter( widget() );

	// draw the titlebar
	QRect title( titlebar_->geometry() );
	QRect titleRight = QRect (title);
	
	title.setWidth( title.width() - 30 );
	painter.drawTiledPixmap( title, QPixmap( t ) );
	
	titleRight.setLeft(  titleRight.left() + titleRight.width() - 30 );
	titleRight.setWidth( 30 );
	painter.drawPixmap(titleRight , QPixmap( tl ) );
	
	// draw title text
	painter.setFont( options() ->font( isActive(), false ) );
	painter.setPen( options() ->color( KDecoration::ColorFont, isActive() ) );
	painter.drawText( title.x() + FRAMESIZE, title.y(),
	                  title.width() - ( FRAMESIZE ), title.height(),
	                  suseCrystalFactory::titleAlign() | AlignVCenter,
	                  caption() );

	// draw frame
	QRect frame( 0, 0, 0, 0 );

	//left
	frame.setRect( 0, 0, FRAMESIZE, height() );
	painter.drawTiledPixmap( frame, QPixmap( l ) );

	//right
	frame.setRect( width() - FRAMESIZE, 0, FRAMESIZE, height() );
	painter.drawTiledPixmap( frame, QPixmap( r ) );

	//buttom
	frame.setRect( 0, height() - FRAMESIZE, width(), ( FRAMESIZE ) );
	painter.drawTiledPixmap( frame, QPixmap( b ) );

	//left buttom
	frame.setRect( 0, height() - FRAMESIZE, FRAMESIZE - 1, FRAMESIZE );
	painter.drawPixmap( frame, QPixmap( cbl ) );

	//right buttom
	frame.setRect( width() - FRAMESIZE + 1, height() - FRAMESIZE, FRAMESIZE - 1, FRAMESIZE );
	painter.drawPixmap( frame, QPixmap( cbr ) );

	//left top
	frame.setRect( 0, 0, FRAMESIZE, TITLESIZE );
	painter.drawPixmap( frame, QPixmap( ctl ) );

	//right top
	frame.setRect( width() - FRAMESIZE, 0, FRAMESIZE, TITLESIZE );
	painter.drawPixmap( frame, QPixmap( ctr ) );
}

//////////////////////////////////////////////////////////////////////////////
// resizeEvent()
// -------------
// Window is being resized

void suseCrystalClient::resizeEvent( QResizeEvent * )
{
	if ( widget() ->isShown() )
	{
		QRegion region = widget() ->rect();
		region = region.subtract( titlebar_->geometry() );
		widget() ->erase( region );
	}
	//widget() ->repaint();
	doShape();
	//repaint(false);
}

//////////////////////////////////////////////////////////////////////////////
// showEvent()
// -----------
// Window is being shown

void suseCrystalClient::showEvent( QShowEvent * )
{
	widget() ->repaint();
	doShape();
}

//////////////////////////////////////////////////////////////////////////////
// maxButtonPressed()
// -----------------
// Max button was pressed

void suseCrystalClient::maxButtonPressed()
{
	if ( button[ ButtonMax ] )
	{
		switch ( button[ ButtonMax ] ->lastMousePress() )
		{
		case MidButton:
			maximize( maximizeMode() ^ MaximizeVertical );
			break;
		case RightButton:
			maximize( maximizeMode() ^ MaximizeHorizontal );
			break;
		default:
			( maximizeMode() == MaximizeFull ) ? maximize( MaximizeRestore )
			: maximize( MaximizeFull );
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// menuButtonPressed()
// -------------------
// Menu button was pressed (popup the menu)

void suseCrystalClient::menuButtonPressed()
{
	if ( button[ ButtonMenu ] )
	{
		QPoint p( button[ ButtonMenu ] ->rect().bottomLeft().x(),
		          button[ ButtonMenu ] ->rect().bottomLeft().y() );
		KDecorationFactory* f = factory();
		showWindowMenu( button[ ButtonMenu ] ->mapToGlobal( p ) );
		if ( !f->exists( this ) )
			return ; // decoration was destroyed
		button[ ButtonMenu ] ->setDown( false );
	}
}

// end suseCrystalClient **********************************************************
//*****************************************************************************



#include "suseCrystalclient.moc"





